var config       = require('./configReader.js')
if(!config.tasks.html) return

var render       = require('gulp-nunjucks-render')
var path         = require('path')
var fs           = require('fs')
var merge        = require('merge');
var fileExists   = require('./fileExists');
var removeBOM    = require('./removeBOM');
var handleErrors = require('./handleErrors');

/**
 * {% component 'name', {title: 'Example', subtitle: 'An example component'} %}
 */
var ComponentTag = function() {
    this.tags = ['component'];

    this.parse = function(parser, nodes, lexer) {
        var token = parser.nextToken();

        var args = parser.parseSignature(null, true);
        parser.advanceAfterBlockEnd(token.value);

        return new nodes.CallExtension(this, 'run', args);
    };

    this.run = function(context, name, data) {
        // components are organized in this manner: components/componentName/componentName.html
        // Note that the component is only given the data object, and the context is not exposed.
        // data ist merged from data files (components/componentName/componentName.json) and the data provided in the component tag
        // data provided in the component tag overrides default content from component
        // check if file exists - otherwise provide empty data
        var dataPath = path.resolve(config.root.src, 'components', name, name + '.json');
        var filePath = path.resolve(config.root.src, 'components', name, name + '.html');
        if (fileExists(dataPath)) {
            var defaultData = {};
            try {
                defaultData = JSON.parse(removeBOM(fs.readFileSync(dataPath, 'utf8')));
            }
            catch (e) {
                handleErrors(e);
            }

            data = merge(true, defaultData, data);
            // could be replaced with the following line to merge recursively - not sure what makes more sense
            //data = merge.recursive(true, defaultData, data);
        }

        var env = render.nunjucks.configure([path.join(config.root.src, config.tasks.html.root)], { autoescaping: false, watch: false });
        env.addExtension('component', new ComponentTag());
        if (config.tasks.html.filters) {
            for (var i = 0; i < config.tasks.html.filters.length; i++) {
                var filterPath = path.resolve(process.cwd(), '../', config.tasks.html.filters[i]);
                var filter = require(filterPath);
                env.addFilter(filter.name, filter.filter);
            }
        }

        return context.env.filters.safe(render.nunjucks.render(filePath, data));
    };

};

module.exports = new ComponentTag();
