var config      = require('../lib/configReader.js');
if(!config.tasks.html) return;

var browserSync  = require('browser-sync');
var data         = require('gulp-data');
var gulp         = require('gulp');
var gulpif       = require('gulp-if');
var gulpcached   = require('gulp-cached');
var handleErrors = require('../lib/handleErrors');
var htmlmin      = require('gulp-htmlmin');
var path         = require('path');
var render       = require('gulp-nunjucks-render');
var fs           = require('fs');
var componentTag = require('../lib/nunjucksExtensionComponent');
var fileExists   = require('../lib/fileExists');
var removeBOM    = require('../lib/removeBOM');
var merge        = require('merge');

// get data for templates
var getData = function(file) {
  var globalData, packageData = {};

  // check for global.json
  var globalDataPath = path.resolve(config.root.src, config.tasks.data.src, 'global.json');
  if (fileExists(globalDataPath)) {
    try {
      globalData = JSON.parse(removeBOM(fs.readFileSync(globalDataPath, 'utf8')));
    }
    catch (e) {
      handleErrors(e);
    }
  }
  // add package json into global data
  var packageDataPath = path.resolve('../../../', 'package.json');
  if (fileExists(packageDataPath)) {
    try {
      packageData = JSON.parse(removeBOM(fs.readFileSync(packageDataPath, 'utf8')));
    }
    catch (e) {
      handleErrors(e);
    }
  }
  packageData = {'package': packageData};
  globalData = merge.recursive(true, packageData, globalData);

  // find relative data file
  var pageDataPath = path.resolve(config.root.src, config.tasks.data.src, file.relative);

  // replace duplicated folder separator
  pageDataPath = pageDataPath.replace('\.html', '.json').replace(path.sep + path.sep, path.sep);

  // remove folders specified in config (so we don't need to put data files into /data/templates
  config.tasks.data.removeFromPath.forEach(function(pathToRemove) {
    pathToRemove = pathToRemove.replace('/', path.sep);
    pageDataPath = pageDataPath.replace(pathToRemove, '');
  });

  // check if file exists - otherwise provide empty data
  var pageData = {};
  if (fileExists(pageDataPath) && !fs.lstatSync(pageDataPath).isDirectory() ) {
    try {
      pageData = JSON.parse(removeBOM(fs.readFileSync(pageDataPath, 'utf8')));
    }
    catch (e) {
      handleErrors(e);
    }
  }

  return merge.recursive(true, globalData, pageData);
}

// this solution appears to only be working if more than one entry is placed in excludeFolders
var exclude = path.normalize('!' + config.root.src + '**/{' + config.tasks.html.excludeFolders.join(',') + '}/**')
var srcPaths = [path.join(config.root.src, config.tasks.html.src), exclude];

// main task
var htmlTask = function() {

  return gulp.src(srcPaths)
      .pipe(data(getData))
      .on('error', handleErrors)
      .pipe(gulpif(!global.production && config.tasks.html.cache, gulpcached('nunjucks-html')))
      .pipe(render({
          // Add node modules to path of gulp-nunjucks in order to be able to use templates
          // from node modules. Example: VIU Styleguide
          path: [path.join(config.root.src, config.tasks.html.root), '../../../node_modules'],
          envOptions: {
          watch: false
        },
        manageEnv: function(env) {
          env.addExtension('component', componentTag);
          if (config.tasks.html.filters) {
            for (var i = 0; i < config.tasks.html.filters.length; i++) {
              var filterPath = path.resolve(process.cwd(), '../', config.tasks.html.filters[i]);
              var filter = require(filterPath);
              env.addFilter(filter.name, filter.filter);
            }
          }
        }
      }))
      .on('error', handleErrors)
      .pipe(gulpif(global.production, htmlmin(config.tasks.html.htmlmin)))
      .pipe(gulp.dest(path.join(config.root.dest, config.tasks.html.dest)))
      .on('end', browserSync.reload);
}

gulp.task('html', htmlTask)
module.exports = htmlTask
